diff mbox

[13/17] qapi-visit: Move error check into gen_visit_fields_call()

Message ID 1455927587-28033-14-git-send-email-eblake@redhat.com
State New
Headers show

Commit Message

Eric Blake Feb. 20, 2016, 12:19 a.m. UTC
When first introduced, neither branch of gen_visit_fields_call()
would output a goto.  But now that the implicit struct visit
always ends with a goto, we should do the same for regular
struct visits, so that callers don't have to worry about whether
they are creating two identical goto's in a row.

Generated code gets slightly larger; if desired, we could patch
qapi.py:gen_visit_fields() to have a mode where it skips the
final goto and leave it up to the callers when to use that mode,
but that adds more maintenance burden when the compiler should
be smart enough to not bloat the .o file just because the .c
file got larger.

Signed-off-by: Eric Blake <eblake@redhat.com>
---
 scripts/qapi-visit.py | 8 ++++----
 1 file changed, 4 insertions(+), 4 deletions(-)
diff mbox

Patch

diff --git a/scripts/qapi-visit.py b/scripts/qapi-visit.py
index 3aca6a7..5d861ba 100644
--- a/scripts/qapi-visit.py
+++ b/scripts/qapi-visit.py
@@ -55,6 +55,7 @@  def gen_visit_fields_call(typ, direct_name, implicit_name=None):
     visit_type_%(c_type)s_fields(v, %(c_name)s, &err);
 ''',
                      c_type=typ.c_name(), c_name=direct_name)
+        ret += gen_err_check()
     return ret


@@ -79,7 +80,6 @@  static void visit_type_%(c_name)s_fields(Visitor *v, %(c_name)s *obj, Error **er

     if base:
         ret += gen_visit_fields_call(base, '(%s *)obj' % base.c_name())
-        ret += gen_err_check()

     ret += gen_visit_fields(members, prefix='obj->')

@@ -111,9 +111,9 @@  static void visit_type_%(c_name)s_fields(Visitor *v, %(c_name)s *obj, Error **er
     }
 ''')

-    # 'goto out' produced for base, by gen_visit_fields() for each member,
-    # and if variants were present
-    if base or members or variants:
+    # 'goto out' produced for non-empty base, by gen_visit_fields() for
+    # each member, and if variants were present
+    if (base and not base.is_empty()) or members or variants:
         ret += mcgen('''

 out: